@@ -83,8 +83,12 @@ on_name_lost (GDBusConnection *connection,
               gpointer    data)
 {
         if (connection == NULL) {
-                g_warning ("Lost name on bus: %s", name);
-                gsm_fail_whale_dialog_we_failed (TRUE, TRUE, NULL);
+                if (gsm_manager_get_dbus_disconnected (manager))
+                        gsm_quit ();
+                else {
+                        g_warning ("Lost name on bus: %s", name);
+                        gsm_fail_whale_dialog_we_failed (TRUE, TRUE, NULL);
+                }
         } else {
                 g_debug ("Calling name lost callback function");
 
